/*
  Expression Killer
    (C) あかつきみさき（みくちぃP）

  このスクリプトについて
    選択したレイヤーのすべてのエクスプレッションを削除するスクリプト.

  使用方法
    レイヤーを選択してスクリプトを実行.

  バージョン情報
    2017/12/24 Ver 1.0.0 Release
*/

/// <reference path="C:\Users\RUI\OneDrive\lib\aftereffects.d.ts\ae.d.ts"/>

(() => {
    const isCompActive = function (comp: CompItem): comp is CompItem {
        if (!(comp && comp instanceof CompItem)) {
            return false;
        } else {
            return true;
        }
    }

    const isLayerSelected = function (layers: Layer[]) {
        if (layers.length === 0) {
            return false;
        } else {
            return true;
        }
    }

    PropertyBase.prototype.isHidden = PropertyGroup.prototype.isHidden = Property.prototype.isHidden = function () {
        const curSelected: boolean = this.selected;
        let isHiddenProperty: boolean = false;

        // hiddenのプロパティは選択ができない
        // 選択に失敗した場合は一行目は実行されない. 成功した場合は二行目で選択状態の復元を行う.
        try {
            this.selected = !curSelected;
            this.selected = curSelected;
        } catch (err) {
            isHiddenProperty = true;
        }

        return isHiddenProperty;
    }

    PropertyBase.prototype.removeAllExpression = PropertyGroup.prototype.removeAllExpression = Property.prototype.removeAllExpression = function () {
        for (let i = 1; i <= this.numProperties; i++) {
            if (this.property(i) instanceof PropertyBase || this.property(i) instanceof PropertyGroup) {
                this.property(i).removeAllExpression();
            }

            if (this.property(i).isHidden()) {
                continue;
            }

            if (this.property(i).canSetExpression && this.property(i).expression != "") {
                this.property(i).expression = "";
            }
        }
        return;
    }

    const actComp: CompItem = <CompItem>app.project.activeItem;
    if (!isCompActive(actComp)) {
        return 0;
    }

    const selLayers: Layer[] = <Layer[]>actComp.selectedLayers;
    if (!isLayerSelected(selLayers)) {
        return 0;
    }

    app.beginUndoGroup("Expression Killer");

    for (let i = 0; i < selLayers.length; i++) {
        for (let j = 1; j <= selLayers[i].numProperties; j++) {
            selLayers[i].property(j).removeAllExpression();
        }
    }

    app.endUndoGroup();

    return 0;
}).call(this);
